'''Data loader module.
'''

import os
import pygame


data_dir = os.path.normpath(os.path.join('data'))


def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)


def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)


def load_image(filename, size=[None,None]):
    '''Loads image returning pygame.image object.
    Complete filename is mandatory argument.
    Size is optional argument.
        If specified, it will be used to resize the output image objects (width,heigth).
        If not specified, image will be loaded with its own dimensionstandard .
    '''    
    #originalImage = pygame.image.load(filepath(filename)).convert()
    originalImage = pygame.image.load(filepath(filename)).convert_alpha()
    if size<>[None,None]:
        scaledImage = pygame.transform.scale(originalImage,[size[0], size[1]])
        scaledImage=scaledImage.convert_alpha()
        return scaledImage
    return originalImage

"""NOT USED
def load_all_images(imageDict={}): # name, width, height
    '''Loads all the images that will be used in the game.
    Images are stored in the data_dir folder as png files.
    Images will be stored in a dictionary of pygame.image named myImages.
    myImages.keys() will be equal to the names of the png files (all chars lower case)
    
    If function is called without any argument, it will load all the images using their own original size.
    Optional argument imageDict is dictionary of image names. Its values are tuple of 2 elements: width and height
    Matched file names in the keys will be loaded with specified sizes.
    '''
    myImages={}
    width=height=None
    for f in os.listdir(data_dir):
        name,ext=f[:-4],f[-3:]
        if ext in ('png','jpg'):
            if name in imageDict.keys():
                width=imageDict[name][0]
                height=imageDict[name][1]
            else:
                myImages[name]=load_image(name+'.'+ext, [width, height])      
        else:
            print f,'is not valid png image'
    return myImages
"""

def load_animation(filename, size=[None,None]):
    '''Loads an image returning a tuple of pygame.image objects.
    Filename should be like r_c_filename.ext where
        r stands for number of frames in a column 
        c stands for number of frames in a row
        ext should be in jpg, png 
    Each frame will be loaded in one element of the tuple.
    Complete filename is mandatory argument.
    Size is optional argument.
        If specified, it will be used to resize the output image objects (width,heigth).
        If not specified, image will be loaded with its own dimensionstandard .
    '''       
    try:
        numOfRows=int(filename[:filename.index('_')])
        numOfCols=int(filename[filename.index('_')+1:filename.index('_',filename.index('_')+1)])
    except ValueError:
        print filename,'is not valid image filename'
        numOfRows,numOfCols=1
    anim = []
    full_image = load_image(filename,size)
    frame_width = full_image.get_width()/numOfCols
    frame_height = full_image.get_height()/numOfRows
    for row in range(numOfRows):
        for col in range(numOfCols):
            singleFrame=full_image.subsurface((col*frame_width,row*frame_height,frame_width,frame_height))
            anim.append(singleFrame)
    return anim


def load_all_animations(animDict={}):
    '''Loads all the animations that will be used in the game.
    Images are stored in the data_dir folder as png files.
    Animations will be stored in a dictionary of pygame.image named myAnims.
    myAnims.keys() will be the names of the png files (all chars lower case)
    myAnims.values() will be tuples of pygame.image, each one containing one fram of the animation
    
    If function is called without any argument, it will load all the images with one single frame.
    Optional argument animDict is dictionary of animation names. Its values are tuple of 2 elements: rows and cols.
    Matched file names in the keys will be loaded with specified number of frames.
    '''        
    myAnims={}
    print 'Loading images...'
    for f in os.listdir(data_dir):
        size=[None,None]
        name,ext=f[:-4],f[-3:]
        if ext in ('png','jpg'):
            anim,name=name[:name.index('_',name.index('_')+1)+1],name[name.index('_',f.index('_')+1)+1:]
            print name
            if name in animDict.keys():
                size=animDict[name]
            myAnims[name]=load_animation(f, size)
    return myAnims

def load_music(filename):
    '''Loads music
    '''
    pygame.mixer.music.load(filepath(filename))
    pygame.mixer.music.set_volume(0.3)

def load_all_sounds():
    '''Loads all the sounds effects that will be used in the game.
    '''
    mySounds={}
    print 'Loading audio...'
    for f in os.listdir(data_dir):
        name,ext=f[:-4],f[-3:]
        if ext in ('ogg'):
            print name
            mySounds[name]=pygame.mixer.Sound(filepath(f))
            mySounds[name].set_volume(0.3)
    return mySounds            

